#include "stdafx.h"
#include <cuda.h>
#include <cutil.h>
#include <back.h>

/*
//Struct.
struct objects{
	float id;
	float positionX;
	float positionY;
	float speedX;
	float speedY;
};

//Struct.
struct save{
	float id;
	float time;
	float closestNeighbour;
};
*/

__global__ void cudaComputation(objects* obj, save* results, int N){
	int i = blockIdx.x * blockIdx.x + threadIdx.x;

	
	//Current object.
//	float currentId = obj[i].id;
	float currentPositionX = obj[i].positionX;
	float currentPositionY = obj[i].positionY;
	float currentSpeedX = obj[i].speedX;
	float currentSpeedY = obj[i].speedY;

	//Variable for keeping the closest distance.
	float Wc = 0;
	float smalestWc = 1000000000;
	float tc;

	for(int j=0; j<N; j++){

		//Berkna W0.
		//Hmta valt objekts (P) pos x och temp objekts (Q) pos x.
		float w0o = (currentPositionX - obj[j].positionX);
		float w0u = (currentPositionY - obj[j].positionY);
		
		//Berkna (p - q).
		float pMinusqOver = (currentSpeedX - obj[j].speedX);
		float pMinusqUnder = (currentSpeedY - obj[j].speedY);
		
		//Berkna |p - q|^2.
		float pMinusQ = (pMinusqOver * pMinusqOver) + (pMinusqUnder * pMinusqUnder);
		
		//Berkna tc, tiden d objekten komme att befinna sig nrmast varandra.
		tc = (-((w0o * pMinusqOver) + (w0u * pMinusqUnder))) / (pMinusQ);
		
		//Berkna vilket avstnd objekten P och Q kommer att ha till varandra vid tid tc.				
		float PtcX = (currentPositionX + (tc * currentSpeedX));
		float PtcY = (currentPositionY + (tc * currentSpeedY));
		
		float QtcX = (obj[j].positionX + (tc * obj[j].speedX));
		float QtcY = (obj[j].positionY + (tc * obj[j].speedY));
		
		//Wc = Ptc - Qtc.
		float WcX = (PtcX - QtcX);
		float WcY = (PtcY - QtcY);
		Wc = sqrt((WcX * WcX) + (WcY * WcY));
		
		if(Wc < smalestWc){
			smalestWc = Wc;
			
//			saveSmallestWc(results, obj, i, j, tc);

			results[i].id = obj[i].id;
			results[i].time = tc;
			results[i].closestNeighbour = obj[j].id;
			
		}// End if.

	}// End for*/
}

void printResults(save* results_h, int N){
	for(int i=0; i<N; i++){
		printf("%s %d %f\n", "ID:       ", i, results_h[i].id);
		printf("%s %d %f\n", "Time:     ", i, results_h[i].time);
		printf("%s %d %f\n\n", "Neighbour:", i, results_h[i].closestNeighbour);
	}
}

void computeCPA(objects* obj_h, save* results_h, int N){

	// Create pointers to arrays.
	objects *obj_d;
	save *results_d;

	// Number of bytes that each array will need.
	size_t obj_size = N * sizeof(objects);
	size_t results_size = N * sizeof(save);

	printf("%d %d", obj_size, results_size);

	// Allocate memory on GPU.
	cudaMalloc((void **) &obj_d, obj_size);
	cudaMalloc((void **) &results_d, results_size);

	// Copy arrays to GPU.
	cudaMemcpy(obj_d, obj_h, obj_size, cudaMemcpyHostToDevice);
	cudaMemcpy(results_d, results_h, results_size, cudaMemcpyHostToDevice);

	//Set block size.
	int blockSize = 64;	// Use the occupancy calculator to find out what size to use.

	//Calculate how many blocks that will be used.
	int numberOfBlocks = blockSize/N + (N%blockSize == 0 ? 0:1);

	//Call function on GPU.
	cudaComputation <<< numberOfBlocks, blockSize >>> (obj_d, results_d, N);

	//Copy the results back to CPU.
	cudaMemcpy(obj_h, obj_d, obj_size, cudaMemcpyDeviceToHost);
	cudaMemcpy(results_h, results_d, results_size, cudaMemcpyDeviceToHost);

	// Print the results.
	printResults(results_h, N);
	backWithResults(results_h, N);

	//cleanup.
	free(obj_h);
	free(results_h);
	cudaFree(obj_d);
	cudaFree(results_d);
}